Package: tsdf
Type: Package
Title: Two-/Three-Stage Designs for Phase 1&2 Clinical Trials
Version: 1.1-9
Date: 2026-04-25
Authors@R: c(
    person("Wenchuan", "Guo", email = "wguo1017@gmail.com", role = c("aut", "cre")),
    person("Jianan", "Hui", email = "jiananhuistat@gmail.com", role = "aut"),
    person("Bob", "Zhong", role = "aut")
  )
Description: Calculates Zhong's optimal two-/three-stage Phase II designs
    for single-arm trials, generates target-toxicity decision tables for
    two-/three-stage Phase I dose-finding, and supports dose-finding
    simulations using custom decision tables. The Phase II design is based on
    Zhong (2012) <doi:10.1016/j.cct.2012.07.006>.
License: GPL-2
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-25 22:45:03 UTC; wguo
Author: Wenchuan Guo [aut, cre],
  Jianan Hui [aut],
  Bob Zhong [aut]
Maintainer: Wenchuan Guo <wguo1017@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-26 00:30:02 UTC
Built: R 4.6.0; ; 2026-04-27 00:12:06 UTC; unix

Package: tsdf
Type: Package
Title: Two-/Three-Stage Designs for Phase 1&2 Clinical Trials
Version: 1.1-9
Date: 2026-04-25
Authors@R: c(
    person("Wenchuan", "Guo", email = "wguo1017@gmail.com", role = c("aut", "cre")),
    person("Jianan", "Hui", email = "jiananhuistat@gmail.com", role = "aut"),
    person("Bob", "Zhong", role = "aut")
  )
Description: Calculates Zhong's optimal two-/three-stage Phase II designs
    for single-arm trials, generates target-toxicity decision tables for
    two-/three-stage Phase I dose-finding, and supports dose-finding
    simulations using custom decision tables. The Phase II design is based on
    Zhong (2012) <doi:10.1016/j.cct.2012.07.006>.
License: GPL-2
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-25 22:45:03 UTC; wguo
Author: Wenchuan Guo [aut, cre],
  Jianan Hui [aut],
  Bob Zhong [aut]
Maintainer: Wenchuan Guo <wguo1017@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-26 00:30:02 UTC
Built: R 4.6.0; ; 2026-05-01 20:32:00 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,dec.sim)
S3method(plot,dec.table)
S3method(print,dec.table)
S3method(print,opt.design)
S3method(print,summary.dec.sim)
S3method(summary,dec.sim)
export(adj.two)
export(dec.sim)
export(dec.table)
export(opt.design)
export(sl.sim)
import(graphics)
import(stats)
import(utils)

tsdf
===

[![](https://www.r-pkg.org/badges/version/tsdf)](https://cran.r-project.org/package=tsdf)
[![CRAN RStudio mirror downloads](https://cranlogs.r-pkg.org/badges/tsdf)](https://cran.r-project.org/package=tsdf)

### What is it?
`tsdf` calculates optimal Zhong's two-/three-stage Phase II designs, generates decision tables for two-/three-stage Phase I dose-finding, and allows users to run dose-finding simulations with custom decision tables.
### Installation
To install from `CRAN`:
```r
install.packages("tsdf")
```

### References
[1] Bob Zhong, Single-arm phase IIA clinical trials with go/no-go decisions, Contemp Clin Trials, 2012

[2] Wenchuan Guo and Bob Zhong, Target toxicity design for phase I dose-finding, Statistical Theory and Related Fields, 2021

#' Adjust Zhong's two-stage Phase II design
#' @description Adjust Zhong's two-stage design for over-running or
#'   under-running.
#' @details This function enumerates feasible second-stage boundaries after the
#'   first-stage sample size and boundaries have been fixed.
#' @param n1 Sample size at stage 1.
#' @param r1 Inefficacy boundary at stage 1.
#' @param s1 Efficacy boundary at stage 1. If there is no early stopping for
#'   efficacy, \code{s1} should equal \code{n1}.
#' @param n2 Sample size at stage 2.
#' @param alpha1 Left-side overall type I error.
#' @param alpha2 right-side overall type I error.
#' @param beta Type II error.
#' @param pc A numeric vector of response rates. It should have length 1 or 2.
#' @param pe Alternative hypothesis response rate.
#' @param ... Unused arguments.
#' @return An object of class \code{"opt.design"}, returned as a list
#'   containing:
#'   \item{bdry}{The rejection boundaries.}
#'   \item{error}{The true type I and type II errors.}
#'   \item{n}{The sample size at each stage.}
#'   \item{complete}{The complete list of feasible designs.}
#'   \item{alpha1}{The input left-side type I error.}
#'   \item{alpha2}{The input right-side type I error.}
#'   \item{beta}{The input type II error.}
#'   \item{pc}{The input response-rate vector.}
#'   \item{pe}{The input alternative response rate.}
#'   \item{stage}{The number of stages in the selected design.}
#' @author Wenchuan Guo <wguo1017@gmail.com>, Jianan Hui <jiananhuistat@gmail.com>
#' @import stats
#' @export
#' @examples 
#'  n1 <- 22
#'  r1 <- 6
#'  s1 <- 22
#'  n2 <- 24
#'  pc <- 0.4
#'  pe <- pc + 0.15
#'  alpha1 <- 0.3
#'  alpha2 <- 0.1
#'  beta <- 0.2
#'  out <- adj.two(n1, r1, s1, n2, alpha1, alpha2, beta, pc, pe)


adj.two <- function(n1, r1, s1, n2, alpha1, alpha2, beta, pc, pe, ...){
  if(length(pc) == 1) {
    pc <- rep(pc, 2)
  }
  L1 <- pbinom(r1, n1, pc[1])
  R1 <- 1 - pbinom(s1, n1, pc[2])
  t1 <- (r1+1) : s1
  db_left <- dbinom(t1, n1, pc[1])
  db_right <- dbinom(t1, n1, pc[2])
  db_pe <- dbinom(t1, n1, pe)
  results <- vector("list", max(0, (s1 + n2) - r1))
  idx <- 0L
  for(r2 in (s1 + n2) : (r1+1)){
    L2 <- L1 + sum(db_left * pbinom(r2 - t1, n2, pc[1]))
    if(L2 <= alpha1) {
      s2_l <- ifelse(s1!=n1, max(r2, s1), r2)
      for(s2 in s2_l : (s1+n2)) {
        R2 <- R1 + sum(db_right * (1 - pbinom(s2 - t1, n2, pc[2])))
        if(R2 <= alpha2) {
          Rpe <- sum(db_pe * (1 - pbinom(s2 - t1, n2, pe))) + 1 - pbinom(s1, n1, pe)
          idx <- idx + 1L
          results[[idx]] <- c(L1, L2, R1, R2, 1 - Rpe, r1, r2, s1, s2, n1, n2)
        } 
        else next
      }
    } else next 
  }
  if(idx == 0L) {
    stop("No feasible design found")
  }
  out <- round(do.call(rbind, results[seq_len(idx)]), 4)
  # out <- out[out[, 5] < (beta + 0.2), ]
  out <- out[order(-out[, 2], -out[, 4], out[, 5]), ]
  out <- as.matrix(do.call(rbind, by(out, out[, 7], FUN=function(x) head(x, 1))))
  colnames(out) <- c("alpha11", "alpha12", "alpha21", "alpha22", "beta", "r1", "r2", "s1", "s2", "n1", "n2")
  out <- out[order(-out[, 2], -out[, 4], out[, 5]), ]
  opt <- out[1, ]
  res <- list(bdry = opt[6:9], error = opt[1:5], n = opt[10:11], complete = out)
  input <- list(alpha1 = alpha1, alpha2 = alpha2, beta = beta, pc = pc, pe = pe, stage = 2)
  res <- c(res, input)
  class(res) <- "opt.design"
  return(res)
}

#' Run dose-finding simulations
#' @description Run dose-finding simulations based on a customized decision
#'   table.
#' @details Assume there are \eqn{d} dose levels to be studied. Let
#'   \eqn{n_i} and \eqn{m_i} denote the cumulative number of patients treated
#'   and the cumulative number of DLTs observed at the current dose level,
#'   respectively. Let \eqn{n_{max}} be the maximum number of patients allowed
#'   at each dose level. The procedure is as follows:
#' \describe{
#' \item{Step 1}{Update the cumulative numbers of DLTs (\eqn{m_i}) and
#' total treated patients (\eqn{n_i}) at the current dose, then use the
#' decision table to choose an action. If the decision is \code{"S"}, go to
#' Step 2. If the decision is \code{"D"} or \code{"DU"}, go to Step 3. If the
#' decision is \code{"E"}, go to Step 4.}
#' \item{Step 2}{If \eqn{n_i = n_{max}}, declare dose \eqn{i} as the MTD.
#' Otherwise, treat an additional cohort at the current dose and return to
#' Step 1.}
#' \item{Step 3}{If the current dose is the lowest dose, stop the trial and
#' declare that the MTD is below the lowest dose. Otherwise, if the next lower
#' dose has not yet reached \eqn{n_{max}}, treat an additional cohort there,
#' move to that dose, and return to Step 1. If the next lower dose has already
#' reached \eqn{n_{max}}, stop the trial and declare that dose as the MTD. If
#' the decision is \code{"DU"}, record the current dose as unusable and do not
#' treat additional patients there.}
#' \item{Step 4}{If the current dose is the highest dose, stop the trial and
#' declare that the MTD is above the highest dose. Otherwise, if the next
#' higher dose has been marked \code{"DU"}, continue treating the current dose
#' until it reaches \eqn{n_{max}}; if that limit is reached, declare the
#' current dose as the MTD. If the next higher dose is available and has not
#' yet reached \eqn{n_{max}}, treat an additional cohort there, move up to
#' that dose, and return to Step 1. Otherwise, declare the current dose as the
#' MTD.}
#' }
#' @param truep A vector of length \eqn{k}, giving the true toxicity
#'   probabilities for the dose levels under study.
#' @param decTable A customized decision table in the same format as the output
#'   of \code{\link{dec.table}}.
#' @param start.level Starting dose level. Defaults to 1, the lowest dose
#'   level.
#' @param nsim Number of simulated trials. Defaults to 1000.
#' @return An object of class \code{"dec.sim"}. Use
#'   \code{\link{summary.dec.sim}} to obtain and print a summary table of the
#'   results. The returned object is a list containing:
#'   \item{mtd}{A vector of dose levels giving the recommended maximum
#'   tolerated dose (MTD) at the end of the trial.}
#'   \item{mtd.prob}{A vector of length \eqn{k} giving the proportion of trials
#'   that selected each dose level as the MTD.}
#'   \item{over.prob}{A vector of length \eqn{k} giving the proportion of
#'   trials that selected each dose level as above the MTD.}
#'   \item{n.patients}{The average number of patients treated at each dose
#'   level.}
#'   \item{dlt}{The average number of DLTs observed at each dose level.}
#'   \item{truep}{The input true toxicity probabilities.}
#'   \item{start.level}{The input starting dose level.}
#'   \item{nsim}{The input number of simulated trials.}
#' @author Wenchuan Guo <wguo1017@gmail.com>
#' @import stats
#' @export
#' @examples
#' truep <- c(0.3, 0.45, 0.5, 0.6)
#' res <- dec.table(0.6,0.4,0.2,0.3,c(3,3,3))
#' out <- dec.sim(truep, as.matrix(res$table), start.level = 2, nsim = 1000)
#' summary(out, pt = 0.3)

dec.sim  <- function(truep, decTable, start.level = 1, nsim = 1000) {
  # initialization
  n_dose <- length(truep)
  input.sample <- as.numeric(colnames(decTable))
  maxn <- nrow(decTable) - 1
  nc <- ncol(decTable)
  if(input.sample[length(input.sample)] != maxn) {
    stop("Please check decision table format")
  }
  add <- c(input.sample[1], diff(input.sample))
  decisions <- as.matrix(decTable)
  storage.mode(decisions) <- "character"
  mtd <- rep("", nsim)
  np <- matrix(0, nsim, n_dose)
  dlt <- matrix(0, nsim, n_dose)
  des <- matrix("", nsim, n_dose)
  for(i in seq_len(nsim)){
    # Doses marked as unavailable for future treatment.
    rm.dose <- rep(FALSE, n_dose)
    # Current dose level.
    dose <- start.level
    # Current stage for each dose level.
    sample.stage <- rep(1, n_dose)
    while(mtd[i] == ""){
      if(sample.stage[dose] > nc) {
        stop("Please check decision table format")
      }
      add.sample <- add[sample.stage[dose]]
      np[i, dose] <- np[i, dose] + add.sample
      dlt[i, dose] <- dlt[i, dose] + sum(rbinom(add.sample, 1, truep[dose]))
      des[i, dose] <- decisions[dlt[i, dose] + 1, sample.stage[dose]]
      sample.stage[dose] <- sample.stage[dose] + 1
      # Stay at the current dose.
      if(des[i, dose] == "S"){
        if(np[i, dose] != maxn) {
          dose <- dose
        } 
        else {
          mtd[i] <- as.character(dose)
        } 
      }
      # Escalate to the next dose.
      if(des[i, dose] == "E") {
        if(dose != n_dose) {
          # Check whether the next dose is still available.
          if(rm.dose[dose + 1]) {
            if(np[i, dose] != maxn) {
              dose <- dose
            } else {
              mtd[i] <- as.character(dose)
            }
          } else {
            if(np[i, dose + 1] != maxn) {
              dose <- dose + 1
            } else {
              mtd[i] <- as.character(dose)
            }
          }
        } else {
          mtd[i] <- "U"
        }
      }  
      # De-escalate to the previous dose.
      if(des[i, dose] == "D") {
        if(dose != 1) {
          if(np[i, dose-1] != maxn) {
            dose <- dose - 1
          } else {
            mtd[i] <- as.character(dose - 1L)
          }
        } else {
          mtd[i] <- "L"
        }
      }
      # De-escalate and mark the current dose as unavailable.
      if(des[i, dose] == "DU") {
        if(dose != 1) {
          # Do not return to this dose again.
          rm.dose[dose] <- TRUE
          if(np[i, dose-1] != maxn) {
            dose <- dose - 1
          } else {
            mtd[i] <- as.character(dose - 1L)
          }
        } else {
          mtd[i] <- "L"
        }
      }
    }		
  }
  # Calculate the MTD selection probability.
  mtd.prob <- vapply(c(as.character(seq_len(n_dose)), "L", "U"), function(ii) mean(mtd == ii), numeric(1))
  # Calculate the probability of selecting doses above the MTD.
  over <- matrix(0, nsim, n_dose)
  over[des == "D" | des == "DU"] <- 1
  over[des == "S" | des == "E"] <- 0.5
  max.over <- apply(over, 1, max)
  for(i in seq_len(nsim)) {
    if(max.over[i] == 1) {
      iid <- which(over[i, ] == 1)[1]
    } else {
      iid <- min(max(which(over[i, ] == 0.5)) + 1, n_dose)
    }
    over[i, iid : n_dose] <- 1
  }
  over[over == 0.5] <- 0
  over.prob <- colMeans(over)
  out <- list(mtd = mtd, mtd.prob = mtd.prob, dlt = colMeans(dlt), n.patients = colMeans(np), over.prob = over.prob, truep = truep, start.level = start.level, nsim = nsim)
  class(out) <- "dec.sim"
  return(out)
}

#' Dose-finding simulations for a list of scenarios
#' @description Run dose-finding simulations based on a customized decision
#'   table for one or more scenarios read from a file.
#' @param decTable A customized decision table in the same format as the output
#'   of \code{\link{dec.table}}.
#' @param file The name of the file containing the scenario definitions. See
#'   \code{\link{read.table}} for details.
#' @param header Logical; if \code{TRUE}, the file contains variable names in
#'   the first line. See \code{\link{read.table}} for details.
#' @param sep The field separator character. Defaults to \code{","}. See
#'   \code{\link{read.table}} for details.
#' @param ... Additional arguments passed to \code{\link{read.table}}.
#' @details In each row of the input file, the parameters must be ordered as
#'   \code{start.level}, \code{nsim}, and \code{truep}. The
#'   dose-finding algorithm is described in \code{\link{dec.sim}}.
#' @return An object of class \code{"dec.sim"} (for one scenario) or
#'   \code{"sl.sim"} (for multiple scenarios). Use \code{\link{summary}} to
#'   obtain and print a summary of the results. The returned object is a list
#'   containing:
#'   \item{mtd}{A vector of dose levels giving the recommended maximum
#'   tolerated dose (MTD) at the end of each trial.}
#'   \item{n.patients}{The average number of patients treated at each dose
#'   level.}
#'   \item{truep}{The input true toxicity probabilities.}
#'   \item{start.level}{The input starting dose level.}
#'   \item{nsim}{The input number of simulated trials.}
#' @author Wenchuan Guo <wguo1017@gmail.com>
#' @import stats
#' @import utils
#' @export
#' @examples
#' dt <- dec.table(0.6,0.4,0.2,0.3,c(3,3,3))
#' test.file <- system.file("extdata", "testS.csv", package = "tsdf")
#' # use a customized decision table
#' table.file <- system.file("extdata", "decTable.csv", package = "tsdf")
#' dec <- read.csv(table.file, row.names = 1, check.names = FALSE)
#' out1 <- sl.sim(as.matrix(dt$table), test.file)
#' out2 <- sl.sim(dec, test.file)

sl.sim <- function(decTable, file, header = TRUE, sep = ",", ...) {
  sl <- read.table(file, header, sep, ...)
  out <- lapply(1:nrow(sl), function(i){
    info <- sl[i, ][!is.na(sl[i, ])]
    n.info <- length(info)
    dec.sim(info[3:n.info], decTable, info[1], info[2])
  })
  if(nrow(sl) > 1) {
    class(out) <- c("sl.sim", "dec.sim")
  } else {
    class(out) <- "dec.sim"
  }
  return(out)
}

#' Zhong's 2- or 3-stage Phase II design
#' @description Calculate the optimal 2- or 3-stage design proposed by Bob
#'   Zhong.
#' @details In the two-stage design, \code{n1} patients are treated in the
#'   first stage. At the end of stage 1, the trial either continues to stage 2
#'   or stops early for inefficacy, depending on the number of observed
#'   responses. If the trial continues, an additional \code{n2} patients are
#'   treated. The three-stage design extends the two-stage design by adding one
#'   interim stage between stages 1 and 2. The left-side rejection region is
#'   defined by \code{response <= r_i} for \code{i = 1, 2, 3}, and the
#'   right-side rejection region is defined by \code{response > s}. An
#'   alpha-spending method is available for both two- and three-stage designs.
#'   \code{opt.design} uses the Hwang-Shih-DeCani spending function; you can
#'   change the definition of \code{HSD} to use a different spending function.
#' @param alpha1 Left-side overall type I error.
#' @param alpha2 right-side overall type I error.
#' @param beta Type II error.
#' @param pc A numeric vector of response rates. It should have length 1 or 2.
#' @param pe Alternative hypothesis response rate.
#' @param stage Either 2 or 3. Defaults to 2.
#' @param stop.eff Logical; if \code{TRUE}, the trial may stop early for
#'   efficacy at an interim analysis.
#' @param frac_n1 Proportion range for \code{n1}. For a two-stage design, the
#'   default is \code{c(0.3, 0.6)}. For a three-stage design, the default is
#'   \code{c(0.2, 0.3)}.
#' @param frac_n2 Proportion range for \code{n2}. Used only for three-stage
#'   designs. Defaults to \code{c(0.2, 0.4)}.
#' @param sf.param A single real value specifying the gamma parameter for the
#'   Hwang-Shih-DeCani spending function. The allowable range is [-40, 40].
#'   Larger values spend more error early and leave less for later stages. For
#'   two-stage designs, the default is \code{NULL} (no alpha-spending). For
#'   three-stage designs, the default is 4.
#' @param show Logical; if \code{TRUE}, the current total sample size is printed
#'   during the search.
#' @param nmax Maximum sample size. Defaults to 100.
#' @param n.choice Stopping criterion for the search over feasible designs. The
#'   search stops once the number of designs exceeds \code{n.choice}.
#' @param ... Unused arguments.
#' @return An object of class \code{"opt.design"}, returned as a list
#'   containing:
#'   \item{bdry}{The rejection boundaries.}
#'   \item{error}{The true type I and type II errors.}
#'   \item{n}{The sample size at each stage.}
#'   \item{complete}{The complete list of feasible designs.}
#'   \item{alpha1}{The input left-side type I error.}
#'   \item{alpha2}{The input right-side type I error.}
#'   \item{beta}{The input type II error.}
#'   \item{pc}{The input response-rate vector.}
#'   \item{pe}{The input alternative response rate.}
#'   \item{sf.param}{The input alpha-spending parameter.}
#'   \item{stage}{The number of stages in the selected design.}
#' @author Wenchuan Guo <wguo1017@gmail.com>, Jianan Hui <jiananhuistat@gmail.com>
#' @references
#' Zhong. (2012) Single-arm Phase IIA clinical trials with go/no-go decisions. \emph{Contemporary Clinical Trials}, \bold{33}, 1272--1279.
#' @import stats
#' @export
#' @examples 
#'  alpha1 <- 0.15
#'  alpha2 <- 0.10
#'  beta <- 0.15
#'  pc <- 0.25
#'  pe <- pc + 0.20
#'  # calculate optimal two-stage design without using alpha-spending
#'  opt.design(alpha1, alpha2, beta, pc, pe, stage=2)
#'  
#'  # calculate optimal two-stage design with Pocock-like spending function 
#'  opt.design(alpha1, alpha2, beta, pc, pe, stage = 2, sf.param = 1)
#'  
#'  # calculate optimal three-stage design with an O'Brien-Fleming-like spending function
#'  opt.design(alpha1, alpha2, beta, pc, pe, stage = 3, sf.param = -4)

opt.design <- function(alpha1, alpha2, beta, pc, pe, stage = 2, stop.eff = FALSE, frac_n1 = NULL, frac_n2 = NULL, sf.param = NULL, show = FALSE, nmax = 100, n.choice = 1, ...) {
  if(stage !=2 & stage !=3){
    stop("only support two and three stage designs")
  }
  if(alpha1 > 1 | alpha1 < 0){
    stop("'alpha1' should between 0 and 1")
  }
  if(alpha2 > 1 | alpha2 < 0){
    stop("'alpha2' should between 0 and 1")
  }
  if(beta > 1 | beta < 0){
    stop("'beta' should between 0 and 1")
  }
  if(stage == 2) {
    if(is.null(frac_n1)) frac_n1 <- c(0.3, 0.6)
    out <- zhong.two(alpha1, alpha2, beta, pc, pe, stop.eff, sf.param, show, nmax, n.choice, frac_n1)
  } 
  if(stage == 3) {
    if(is.null(sf.param)) sf.param <- 4
    if(is.null(frac_n1)) frac_n1 <- c(0.2, 0.3)
    if(is.null(frac_n2)) frac_n2 <- c(0.2, 0.4)
    out <- zhong.three(alpha1, alpha2, beta, pc, pe, frac_n1, frac_n2, sf.param, stop.eff, show, nmax)
  }
  input <- list(alpha1 = alpha1, alpha2 = alpha2, beta = beta, pc = pc, pe = pe, sf.param = sf.param, stage = stage, frac_n1 = frac_n1, frac_n2 = frac_n2)
  out <- c(out, input)
  class(out) <- "opt.design"
  return(out)
}

#' @keywords internal
convolve_pmf <- function(lhs, rhs) {
  out <- numeric(length(lhs) + length(rhs) - 1L)
  rhs_len <- length(rhs)
  nz <- which(lhs != 0)
  for(i in nz) {
    out[i:(i + rhs_len - 1L)] <- out[i:(i + rhs_len - 1L)] + lhs[i] * rhs
  }
  out
}

#' @keywords internal
stage_total_pmf_from_lookup <- function(t1, stage1_pmf, stage2_pmf) {
  start <- t1[1L]
  stage2_len <- length(stage2_pmf)
  out <- numeric(t1[length(t1)] - start + stage2_len)
  weights <- stage1_pmf[t1 + 1L]
  for(i in seq_along(t1)) {
    offset <- t1[i] - start + 1L
    out[offset:(offset + stage2_len - 1L)] <- out[offset:(offset + stage2_len - 1L)] + weights[i] * stage2_pmf
  }
  list(start = start, pmf = out, total = sum(weights))
}

#' @keywords internal
stage_total_pmf <- function(t1, n1, prob1, n2, prob2) {
  pmf1 <- numeric(n1 + 1L)
  pmf1[t1 + 1L] <- dbinom(t1, n1, prob1)
  list(start = 0L, pmf = convolve_pmf(pmf1, dbinom(0:n2, n2, prob2)))
}

#' @keywords internal
select_total_pmf <- function(pmf, totals, start = 0L) {
  out <- numeric(max(totals) - min(totals) + 1L)
  idx <- totals - start + 1L
  valid <- idx >= 1L & idx <= length(pmf)
  if(any(valid)) {
    pos <- totals[valid] - min(totals) + 1L
    out[pos] <- out[pos] + pmf[idx[valid]]
  }
  list(start = min(totals), pmf = out, total = sum(out))
}

#' @keywords internal
boundary_cdf_from_pmf <- function(pmf, start, boundaries) {
  cdf <- cumsum(pmf)
  idx <- boundaries - start + 1L
  out <- numeric(length(boundaries))
  valid <- idx >= 1L
  if(any(valid)) {
    out[valid] <- cdf[pmin(idx[valid], length(cdf))]
  }
  out
}

#' @keywords internal
boundary_survival_from_pmf <- function(pmf, start, boundaries, total = sum(pmf)) {
  total - boundary_cdf_from_pmf(pmf, start, boundaries)
}

#' @keywords internal
three.opt <- function(alpha1, alpha2, pt, n, sf.param, pe.par, ...){
  # initialization
  nc <- cumsum(n)
  nt <- nc[3]
  as_left <- HSD(alpha1, nc/nt, sf.param)
  as_right <- HSD(alpha2, nc/nt, sf.param)
  # boundary of r1: [0, n1]
  r1_bdry <- 0:nc[1]
  out_r1 <- unique(pbinom(r1_bdry, n[1], pt[1]))
  ind_r1 <- which(out_r1 <= as_left[1])
  # check if conditions hold
  if(length(ind_r1)==0) stop("No optimal design (left side)")
  r1 <- r1_bdry[max(ind_r1)]
  out_r1 <- out_r1[max(ind_r1)]
  # boundary of s1 (r1, n1-1]
  s1_bdry <- r1:(n[1]-1)
  out_s1 <- unique(1 - pbinom(s1_bdry, n[1], pt[2]))
  ind_s1 <- which(out_s1 <= as_right[1])
  # check if condition holds 
  if(length(ind_s1) == 0) stop("No optimal design (right side)")
  s1 <- s1_bdry[min(ind_s1)]
  out_s1 <- out_s1[min(ind_s1)]
  # boundary of r1: [r1, n1+n2-1]
  r2_bdry <- r1:(nc[2]-1)
  t1 <- (r1+1):s1
  left12 <- stage_total_pmf(t1, n[1], pt[1], n[2], pt[1])
  right12 <- stage_total_pmf(t1, n[1], pt[2], n[2], pt[2])
  out_r2 <- boundary_cdf_from_pmf(left12$pmf, left12$start, r2_bdry) + out_r1
  out_r2 <- unique(out_r2)
  ind_r2 <- which(out_r2 <= as_left[2])
  # check if conditions hold
  if(length(ind_r2) == 0) stop("No optimal design (left side)")
  r2 <- r2_bdry[max(ind_r2)]
  out_r2 <- out_r2[max(ind_r2)]
  # boundary of s2
  s2_bdry <- r2:nc[2]
  out_s2 <- boundary_survival_from_pmf(right12$pmf, right12$start, s2_bdry) + out_s1
  out_s2 <- unique(out_s2)
  ind_s2 <- which(out_s2 <= as_right[2])
  # check if conditions hold
  if(length(ind_s2) == 0) stop("No optimal design (right side)")
  s2 <- s2_bdry[min(ind_s2)]
  out_s2 <- out_s2[min(ind_s2)]
  # boundary of r3: [r2, n1+n2+n3=n]
  r3_bdry <- r2:nc[3]
  left23 <- select_total_pmf(left12$pmf, seq.int(r2 + 1L, s2))
  out_r3 <- boundary_cdf_from_pmf(convolve_pmf(left23$pmf, dbinom(0:n[3], n[3], pt[1])), left23$start, r3_bdry) + out_r2
  out_r3 <-unique(out_r3)
  ind_r3 <- which(out_r3 <= as_left[3])
  # check if conditions hold
  if(length(ind_r3) == 0) stop("No optimal design (left side)")
  r3 <- r3_bdry[max(ind_r3)]
  out_r3 <- out_r3[max(ind_r3)]
  # boundary of s3
  s3_bdry <- r3:(nc[3]-1)
  right23 <- select_total_pmf(right12$pmf, seq.int(r2 + 1L, s2))
  out_s3 <- boundary_survival_from_pmf(convolve_pmf(right23$pmf, dbinom(0:n[3], n[3], pt[2])), right23$start, s3_bdry) + out_s2
  out_s3 <- unique(out_s3)
  ind_s3 <-  which(out_s3 <= as_right[3])
  # check if conditions hold
  if(length(ind_s3) == 0) stop("No optimal design (right side)")
  s3 <- s3_bdry[min(ind_s3)]
  out_s3 <- out_s3[min(ind_s3)]
  # save feasible designs & errors
  bdry <- c(r1, r2, r3, s1, s2, s3)
  err <- c(out_r1, out_r2, out_r3, out_s1, out_s2, out_s3)
  pe <- pt[2] + pe.par
  power12 <- stage_total_pmf(t1, n[1], pe, n[2], pe)
  power23 <- select_total_pmf(power12$pmf, seq.int(r2 + 1L, s2))
  emp_power <- 1 - pbinom(s1, n[1], pe) +
    boundary_survival_from_pmf(power12$pmf, power12$start, s2) +
    boundary_survival_from_pmf(convolve_pmf(power23$pmf, dbinom(0:n[3], n[3], pe)), power23$start, s3)
  # merge results
  names(bdry) <- c("r1", "r2", "r3", "s1", "s2", "s3")
  names(err) <- c("alpha11", "alpha12", "alpha13", "alpha21", "alpha22", "alpha23")
  out <- list(bdry = bdry, error = err, pt = pt, n = n, alpha = c(alpha1, alpha2), beta = 1 - emp_power, sf.param = sf.param)
  class(out) <- "2opt"
  return(out)
}

#' @keywords internal
two.opt <- function(alpha1, alpha2, pt, n, sf.param, pe.par, ...){
  # initialization
  nc <- cumsum(n)
  nt <- nc[2]
  as_left <- HSD(alpha1, nc/nt, sf.param)
  as_right <- HSD(alpha2, nc/nt, sf.param)
  comb <- NULL
  err <- NULL
  # boundary of r1: [0, n1]
  r1_bdry <- 0:nc[1]
  out_r1 <- unique(pbinom(r1_bdry, n[1], pt[1]))
  ind_r1 <- which(out_r1 <= as_left[1])
  # check if conditions hold
  if(length(ind_r1)==0) stop("No optimal design (left side)")
  r1 <- r1_bdry[max(ind_r1)]
  out_r1 <- out_r1[max(ind_r1)]
  # boundary of s1 (r1, n1-1]
  s1_bdry <- r1:(n[1]-1)
  out_s1 <- unique(1 - pbinom(s1_bdry, n[1], pt[2]))
  ind_s1 <- which(out_s1 <= as_right[1])
  # check if condition holds 
  if(length(ind_s1) == 0) stop("No optimal design (right side)")
  s1 <- s1_bdry[min(ind_s1)]
  out_s1 <- out_s1[min(ind_s1)]
  # boundary of r1: [r1, n1+n2-1]
  r2_bdry <- r1:(nc[2]-1)
  t1 <- (r1+1):s1
  left12 <- stage_total_pmf(t1, n[1], pt[1], n[2], pt[1])
  right12 <- stage_total_pmf(t1, n[1], pt[2], n[2], pt[2])
  out_r2 <- boundary_cdf_from_pmf(left12$pmf, left12$start, r2_bdry) + out_r1
  out_r2 <- unique(out_r2)
  ind_r2 <- which(out_r2 <= as_left[2])
  # check if conditions hold
  if(length(ind_r2) == 0) stop("No optimal design (left side)")
  r2 <- r2_bdry[max(ind_r2)]
  out_r2 <- out_r2[max(ind_r2)]
  # boundary of s2
  s2_bdry <- r2:nc[2]
  out_s2 <- boundary_survival_from_pmf(right12$pmf, right12$start, s2_bdry) + out_s1
  out_s2 <- unique(out_s2)
  ind_s2 <- which(out_s2 <= as_right[2])
  # check if conditions hold
  if(length(ind_s2) == 0) stop("No optimal design (right side)")
  s2 <- s2_bdry[min(ind_s2)]
  out_s2 <- out_s2[min(ind_s2)]
  bdry <- c(r1, r2, s1, s2)
  # calculate type-2 error with pt = pt + 0.2
  pe <- pt[2] + pe.par
  err <- c(out_r1, out_r2, out_s1, out_s2)
  power12 <- stage_total_pmf(t1, n[1], pe, n[2], pe)
  emp_power <- 1 - pbinom(s1, n[1], pe) + boundary_survival_from_pmf(power12$pmf, power12$start, s2)
  # merge results
  names(bdry) <- c("r1", "r2", "s1", "s2")
  names(err) <- c("alpha11", "alpha12", "alpha21", "alpha22")
  out <- list(bdry= bdry, error = err, pt = pt, n = n, alpha = c(alpha1, alpha2), beta = 1 - emp_power, sf.param = sf.param)
  class(out) <- "2opt"
  return(out)
}

#' @keywords internal
right.two.opt <- function(alpha, pt, n, sf.param, ...){
  # initialization
  nc <- cumsum(n)
  nt <- nc[2]
  as_right <- HSD(alpha, nc/nt, sf.param)
  # boundary of s1 (0, n1-1]
  s1_bdry <- 0:(n[1]-1)
  out_s1 <- 1-pbinom(s1_bdry, n[1], pt)
  ind_s1 <- which(out_s1 <= as_right[1])
  # check if condition holds 
  if(length(ind_s1)==0) stop("No optimal design (right side)")
  s1 <- s1_bdry[min(ind_s1)]
  out_s1 <- out_s1[min(ind_s1)]
  t1 <- 0:s1
  # boundary of s2
  s2_bdry <- s1:(nc[2]-1)
  right12 <- stage_total_pmf(t1, n[1], pt, n[2], pt)
  out_s2 <- boundary_survival_from_pmf(right12$pmf, right12$start, s2_bdry) + out_s1
  out_s2 <- unique(out_s2)
  ind_s2 <- which(out_s2 <= as_right[2])
  # check if conditions hold
  if(length(ind_s2) == 0) stop("No optimal design (right side)")
  s2 <- s2_bdry[min(ind_s2)]
  out_s2 <- out_s2[min(ind_s2)]
  bdry <- c(s1, s2)
  err <- c(out_s1, out_s2)
  # merge results
  names(bdry) <- c("s1", "s2")
  names(err) <- c("alpha11", "alpha12")
  out <- list(bdry = bdry, error = err, pt = pt, n = n, sf.param = sf.param, alpha = alpha)
  class(out) <- "1opt"
  return(out)
}

#' @keywords internal
right.three.opt <- function(alpha, pt, n, sf.param, ...){
  # initialization
  nc <- cumsum(n)
  nt <- nc[3]
  as_right <- HSD(alpha, nc/nt, sf.param)
  # boundary of s1 (0, n1-1]
  s1_bdry <- 0:(n[1]-1)
  out_s1 <- 1-pbinom(s1_bdry, n[1], pt)
  ind_s1 <- which(out_s1 <= as_right[1])
  # check if condition holds 
  if(length(ind_s1)==0) stop("No optimal design (right side)")
  s1 <- s1_bdry[min(ind_s1)]
  out_s1 <- out_s1[min(ind_s1)]
  # loop over all s1 
  t1 <- 0:s1
  # boundary of s2
  s2_bdry <- s1:(nc[2]-1)
  right12 <- stage_total_pmf(t1, n[1], pt, n[2], pt)
  out_s2 <- boundary_survival_from_pmf(right12$pmf, right12$start, s2_bdry) + out_s1
  out_s2 <- unique(out_s2)
  ind_s2 <- which(out_s2 <= as_right[2])
  # check if conditions hold
  if(length(ind_s2) == 0) stop("No optimal design (right side)")
  s2 <- s2_bdry[min(ind_s2)]
  out_s2 <- out_s2[min(ind_s2)]
  # boundary of s3
  s3_bdry <- s2:(nc[3]-1)
  right23 <- select_total_pmf(right12$pmf, seq.int(0L, s2))
  out_s3 <- boundary_survival_from_pmf(convolve_pmf(right23$pmf, dbinom(0:n[3], n[3], pt)), right23$start, s3_bdry) + out_s2
  out_s3 <- unique(out_s3)
  ind_s3 <-  which(out_s3 <= as_right[3])
  # check if conditions hold
  if(length(ind_s3) == 0) stop("No optimal design (right side)")
  s3 <- s3_bdry[min(ind_s3)]
  out_s3 <- out_s3[min(ind_s3)]
  # save feasible designs & errors
  bdry <- c(s1, s2, s3)
  err <- c(out_s1, out_s2, out_s3)
  # merge results
  names(bdry) <- c("s1", "s2", "s3")
  names(err) <- c("alpha11", "alpha12", "alpha13")
  out <- list(bdry = bdry, error = err, pt = pt, n = n, sf.param = sf.param, alpha = alpha)
  class(out) <- "1opt"
  return(out)
}

#' @keywords internal
zhong.three <- function(alpha1, alpha2, beta, pc, pe, frac_n1 = c(0.1, 0.3), frac_n2 = c(0.2,0.4), sf.param = 1, stop.eff = FALSE, show = TRUE, nmax = 100, ...) {
  if(length(pc) == 1) {
    pc <- rep(pc, 2)
  }
  if(length(alpha1) == 1) {
    alpha1_ini <- c(1, 1, alpha1)
    alpha2_ini <- c(1, 1, alpha2)
  } else {
    stop("alpha1 and alpha2 should be a single value")
  }
  n_count <- 0
  for(nt in 3:nmax){
    results <- list()
    idx <- 0L
    n1_bdry <- floor(nt*frac_n1[1]):ceiling(nt*frac_n1[2])
    n2_bdry <- floor(nt*frac_n2[1]):ceiling(nt*frac_n2[2]) 
    # remove n1=0, n2=0, from boundary 
    n1_bdry <- n1_bdry[n1_bdry!=0]
    n2_bdry <- n2_bdry[n2_bdry!=0]
    n1_n2 <- expand.grid(n1_bdry, n2_bdry)
    # remove n3=0
    sum_n1_n2 <- rowSums(n1_n2)
    id_rm <- sum_n1_n2 < nt 
    n3_bdry <- nt-sum_n1_n2[id_rm]
    n <- as.matrix(cbind(n1_n2[id_rm,], n3_bdry))
    colnames(n) <- c("n1", "n2", "n3")
    for(i in 1:nrow(n)){
      n1 <- n[i, 1]
      n2 <- n[i, 2]
      n3 <- n[i, 3]
      if(is.null(sf.param)) {
        alpha1 <- alpha1_ini
        alpha2 <- alpha2_ini
      }	else {
        alpha1 <- HSD(alpha1_ini[3], cumsum(n[i, ])/nt, sf.param)
        alpha2 <- HSD(alpha2_ini[3], cumsum(n[i, ])/nt, sf.param)
      }
      for(r1 in (n1-1):0) {
        if(stop.eff) s1_lb <- r1 + 1
        else s1_lb <- n1
        # check L1 
        L1 <- pbinom(r1, n1, pc[1])
        if(L1 <= alpha1[1] & L1 <= alpha1[3]){
          for(s1 in s1_lb: n1){
            R1 <- 1 - pbinom(s1, n1, pc[2])
            if(R1 <= alpha2[1] & R1 <= alpha2[3]) {
              t1 <- (r1+1) : s1
              left12 <- stage_total_pmf(t1, n1, pc[1], n2, pc[1])
              right12 <- stage_total_pmf(t1, n1, pc[1], n2, pc[2])
              power12 <- stage_total_pmf(t1, n1, pe, n2, pe)
              pmf3_left <- dbinom(0:n3, n3, pc[1])
              pmf3_right <- dbinom(0:n3, n3, pc[2])
              pmf3_power <- dbinom(0:n3, n3, pe)
              stage1_power <- 1 - pbinom(s1, n1, pe)
              for(r2 in (s1+n2) : r1){
                L2 <- L1 + boundary_cdf_from_pmf(left12$pmf, left12$start, r2)
                if(L2 <= alpha1[2] & L2 <= alpha1[3]) {
                  if(stop.eff) {
                    s2_lb <- max(r2, s1)
                  } 
                    else {
                      s2_lb <- s1+n2
                    } 
                  for(s2 in s2_lb : (s1+n2)) {
                    R2 <- R1 + boundary_survival_from_pmf(right12$pmf, right12$start, s2)
                    if(R2 <= alpha2[2] & R2 <= alpha2[3]) {
                      left23 <- select_total_pmf(left12$pmf, seq.int(r2 + 1L, s2))
                      power23 <- select_total_pmf(power12$pmf, seq.int(r2 + 1L, s2))
                      r3_bdry <- (n3 + s2) : r2
                      l3_vals <- L2 + boundary_cdf_from_pmf(convolve_pmf(left23$pmf, pmf3_left), left23$start, r3_bdry)
                      s3_all <- r2:(s2 + n3)
                      right3_vals <- R2 + boundary_survival_from_pmf(convolve_pmf(left23$pmf, pmf3_right), left23$start, s3_all)
                      power3_vals <- stage1_power +
                        boundary_survival_from_pmf(power12$pmf, power12$start, s2) +
                        boundary_survival_from_pmf(convolve_pmf(power23$pmf, pmf3_power), power23$start, s3_all)
                      for(rr in seq_along(r3_bdry)){
                        r3 <- r3_bdry[rr]
                        L3 <- l3_vals[rr]
                        if(L3 <= alpha1[3]){
                          if(stop.eff) s3_lb <- max(r3, s2)
                          else s3_lb <- r3
                          start_idx <- s3_lb - r2 + 1L
                          for(ss in start_idx:length(s3_all)){
                            s3 <- s3_all[ss]
                            R3 <- right3_vals[ss]
                            if(R3 <= alpha2[3]){
                              Rpe <- power3_vals[ss]
                            } else next
                            if(Rpe >= 1-beta){
                              idx <- idx + 1L
                              results[[idx]] <- c(L1, L2, L3, R1, R2, R3, 1 - Rpe, r1, r2, r3, s1, s2, s3, n1, n2, n3)
                            } else break
                          }
                        } else next
                      } 
                    } else next
                  }
                } else next
              } 
            } else next
          }  
        } else next
      } 
    }
    if(idx > 0L) break
    # if(n_count > n.ratio * n) break
    # if(n_count >= 1) break
    if(show) print(paste("current sample size is", nt))
  }
  out <- do.call(rbind, rev(results))
  colnames(out) <- c("alpha11", "alpha12", "alpha13", "alpha21", "alpha22", "alpha23", "beta", "r1", "r2", "r3", "s1", "s2", "s3", "n1", "n2", "n3")
  # out <- out[order(-out[,3], -out[,6], out[,7]), ]
  out <- out[order(-out[,1], -out[,2], -out[,3], -out[,4], -out[,5], -out[,6], out[,7]), ]
  bdry <- out[1, ][8:13]
  error <- out[1, ][1:7]
  nn <- out[1, ][14:16]
  names(bdry) <- c("r1", "r2", "r3", "s1", "s2", "s3")
  names(nn) <- c("n1", "n2", "n3")
  names(error) <- c("alpha11", "alpha12", "alpha13", "alpha21", "alpha22", "alpha23", "beta")
  return(list(bdry = bdry, error = error, n = nn, complete = out))
}



#' @keywords internal
zhong.two <- function(alpha1, alpha2, beta, pc, pe, stop.eff, sf.param, show, nmax, n.choice, frac_n1,...){
  if(length(pc) == 1) {
    pc <- rep(pc, 2)
  }
  if(length(alpha1) == 1) {
    alpha1_ini <- c(1, alpha1)
    alpha2_ini <- c(1, alpha2)
  } else {
    stop("alpha1 and alpha2 should be a single value")
  }
  
  for(n in 2 : nmax) {
    results <- list()
    idx <- 0L
    n_count <- 0
    n1_bdry <- max(1, floor(n*frac_n1[1])):ceiling(n*frac_n1[2])
    for(n1 in n1_bdry) {
      n2 <- n - n1
      pmf2_left <- dbinom(0:n2, n2, pc[1])
      pmf2_right <- dbinom(0:n2, n2, pc[2])
      pmf2_power <- dbinom(0:n2, n2, pe)
      cdf1_left <- pbinom(0:n1, n1, pc[1])
      tail1_right <- 1 - pbinom(0:n1, n1, pc[2])
      tail1_power <- 1 - pbinom(0:n1, n1, pe)
      db1_left <- dbinom(0:n1, n1, pc[1])
      db1_right <- dbinom(0:n1, n1, pc[2])
      db1_power <- dbinom(0:n1, n1, pe)
      if(!is.null(sf.param)) {
        alpha1 <- HSD(alpha1_ini[2], c(n1, n)/n, sf.param)
        alpha2 <- HSD(alpha2_ini[2], c(n1, n)/n, sf.param)
      } else {
        alpha1 <- alpha1_ini
        alpha2 <- alpha2_ini
      }
      for(r1 in (n1-1):0) {
        if(stop.eff) s1_lb <- r1 + 1
        else  s1_lb <- n1
        # check L1 
        L1 <- cdf1_left[r1 + 1L]
        if(L1 <=  alpha1[1] & L1 <= alpha1[2]){
          for(s1 in s1_lb: n1){
            R1 <- tail1_right[s1 + 1L]
            if(R1 <= alpha2[1] & R1 <= alpha2[2]) {
              t1 <- (r1+1) : s1
              left12 <- stage_total_pmf_from_lookup(t1, db1_left, pmf2_left)
              right12 <- stage_total_pmf_from_lookup(t1, db1_right, pmf2_right)
              power12 <- stage_total_pmf_from_lookup(t1, db1_power, pmf2_power)
              left12_cdf <- cumsum(left12$pmf)
              right12_tail <- right12$total - cumsum(right12$pmf)
              power12_tail <- power12$total - cumsum(power12$pmf)
              stage1_power <- tail1_power[s1 + 1L]
              r2_vals <- (s1 + n2):r1
              r2_idx <- r2_vals - left12$start + 1L
              l2_add <- numeric(length(r2_vals))
              valid_r2 <- r2_idx >= 1L
              l2_add[valid_r2] <- left12_cdf[pmin(r2_idx[valid_r2], length(left12_cdf))]
              L2_vals <- L1 + l2_add
              feasible_r2 <- which(L2_vals <= alpha1[2])
              if(length(feasible_r2) == 0L) {
                next
              }
              boundary_vals <- r1:(s1 + n2)
              boundary_idx_right <- boundary_vals - right12$start + 1L
              boundary_idx_power <- boundary_vals - power12$start + 1L
              r2_add <- numeric(length(boundary_vals))
              rpe_add <- numeric(length(boundary_vals))
              valid_right <- boundary_idx_right >= 1L
              valid_power <- boundary_idx_power >= 1L
              r2_add[valid_right] <- right12_tail[pmin(boundary_idx_right[valid_right], length(right12_tail))]
              rpe_add[valid_power] <- power12_tail[pmin(boundary_idx_power[valid_power], length(power12_tail))]
              R2_all <- R1 + r2_add
              Rpe_all <- stage1_power + rpe_add
              for(rr in feasible_r2){
                r2 <- r2_vals[rr]
                L2 <- L2_vals[rr]
                if(stop.eff) {
                  s2_l <- max(r2, s1)
                } else {
                  s2_l <- r2
                }
                pos <- seq.int(s2_l - r1 + 1L, length(boundary_vals))
                alpha_ok <- which(R2_all[pos] <= alpha2[2])
                if(length(alpha_ok) == 0L) {
                  next
                }
                pos <- pos[alpha_ok[1L]:length(pos)]
                power_ok <- Rpe_all[pos] >= 1 - beta
                keep_len <- match(FALSE, power_ok)
                if(is.na(keep_len)) {
                  keep_pos <- pos
                } else if(keep_len == 1L) {
                  next
                } else {
                  keep_pos <- pos[seq_len(keep_len - 1L)]
                }
                idx <- idx + 1L
                results[[idx]] <- cbind(
                  L1,
                  L2,
                  R1,
                  R2_all[keep_pos],
                  1 - Rpe_all[keep_pos],
                  r1,
                  r2,
                  s1,
                  boundary_vals[keep_pos],
                  n1,
                  n2
                )
              }
            } else next
          }
        } else next
      }
    }
    if(idx > 0L) {
      out <- do.call(rbind, results)
      if(stop.eff){
        en <- apply(out, 1, function(x) x[10] + x[11]*(1-x[3]-x[1]))
      } else {
        en <- apply(out, 1, function(x) x[10] + x[11]*(1-x[1]))
      }
      out <- cbind(out, en)
      out <- round(out, 4)
      out <- out[order(out[,10], -out[, 2], -out[, 4], out[, 5], -out[, 1], -out[, 3], -out[, 8], -out[, 9]), ]
      out <- do.call(rbind, by(out, out[, 10], FUN=function(x) head(x, 1)))
      n_count <- nrow(out)
    }
    if(n_count > n.choice) break
    if(show) print(paste("current sample size is", n))
  }
  out <- as.matrix(out)
  out <- out[order(out[,10], -out[, 2], -out[, 4], -out[, 5]), ]
  colnames(out) <- c("alpha11", "alpha12", "alpha21", "alpha22", "beta", "r1", "r2", "s1", "s2", "n1", "n2", "EN")
  opt <- out[which.min(out[, 12]), ]
  return(list(bdry = opt[6:9], error = opt[1:5], n = opt[10:11], complete = out))
}





#' @keywords internal
HSD <- function (alpha, t, param) {
  t[t > 1] <- 1
  spend <- if (param == 0) t * alpha else alpha * (1 - exp(-t * param))/(1 - exp(-param))
  return(spend)
}

#' Generate a dose-finding decision table
#' @description Generate a two- or three-stage dose-finding decision table.
#' @details An alpha-spending method is available for two- and three-stage
#'   designs. \code{dec.table} uses the Hwang-Shih-DeCani spending function.
#' @param alpha.l Left-side overall type I error. This controls the upper bound
#'   for dose escalation.
#' @param alpha.r Right-side overall type I error. This controls the lower bound
#'   for dose de-escalation.
#' @param alpha.u Right-side overall type I error used to determine the lower
#'   bound for \code{"DU"}.
#' @param pt A numeric vector of target toxicities. It should have length 1, or
#'   length 2 when the target is an interval.
#' @param n A vector of sample sizes at each stage. \code{sum(n)} is the total
#'   sample size. For A+B designs, \code{n} has length 2; for A+B+C designs,
#'   \code{n} has length 3.
#' @param sf.param A single real value specifying the gamma parameter for the
#'   Hwang-Shih-DeCani spending function. The allowable range is [-40, 40].
#'   Larger values spend more error early and leave less for later stages.
#'   Defaults to 4.
#' @param pe.par Alternative hypothesis offset used to calculate power and the
#'   type II error. The alternative is defined as \code{pe = pt + pe.par}.
#'   Defaults to 0.25.
#' @param ... Unused arguments.
#' @return An object of class \code{"dec.table"}, returned as a list
#'   containing:
#'   \item{table}{The generated decision table.}
#'   \item{alpha.two}{A vector of true type I errors for the two-sided test.}
#'   \item{alpha.one}{A vector of true type I errors for the right-sided test.}
#'   \item{beta}{The true type II error, which depends on the alternative
#'   hypothesis.}
#'   \item{E}{A vector of \code{"E"} boundaries.}
#'   \item{D}{A vector of \code{"D"} boundaries.}
#'   \item{DU}{A vector of \code{"DU"} boundaries.}
#'   \item{pt}{The input target toxicity vector.}
#'   \item{n}{The input stage-wise sample sizes.}
#'   \item{sf.param}{The input alpha-spending parameter.}
#' @author Wenchuan Guo <wguo1017@gmail.com>
#' @import stats
#' @export
#' @examples
#' alpha.l <- 0.6
#' alpha.r <- 0.4
#' alpha.u <- 0.1
#' pt <- 0.3
#' # Print the decision table for a 3+3+3 design
#' n <- rep(3, 3)
#' dec.table(alpha.l, alpha.r, alpha.u, pt, n)$table
#' # 3+3 design
#' n <- rep(3, 2)
#' dec.table(alpha.l, alpha.r, alpha.u, pt, n)$table

dec.table <- function(alpha.l, alpha.r, alpha.u, pt, n, sf.param  = 4, pe.par = 0.25, ...) {
  # check
  err <- c(alpha.l, alpha.r, alpha.u)
  k <- length(n)
  if(sum(err < 0 | err > 1) != 0) {
    stop("'alpha' should between 0 and 1.")
  }
  if(length(pt) > 2) {
    stop("'pt''s length should less than 2 (two-tailed test).")
  }
  if(length(pt) == 1) {
    pt <- rep(pt, 2)
  }
  if(k != 3 & k != 2) {
    stop("This function only find two-stage/three-stage optimal design")
  }
  if(k == 3) {
    out.two <- three.opt(alpha.l, alpha.r, pt, n, sf.param, pe.par)
    out.one <- right.three.opt(alpha.u, pt[2], n, sf.param)
  } else {
    out.two <- two.opt(alpha.l, alpha.r, pt, n, sf.param, pe.par)
    out.one <- right.two.opt(alpha.u, pt[2], n, sf.param)
  }
  des <- list(E = out.two$bdry[1:k], D = out.two$bdry[(k+1):(2*k)], DU = out.one$bdry,  n = n, pt = pt, sf.param = sf.param, alpha.two = out.two$error, alpha.one = out.one$error, beta = out.two$beta)
  r <- des$E
  s <- des$D
  su <- des$DU
  ns <- length(n)
  nt <- sum(n)
  nc <- cumsum(n)
  ans <- matrix(0, nt+1, ns)
  rownames(ans) <- 0:nt
  colnames(ans) <- nc
  for(j in 1:length(r)){
    ans[ ,j][1:(r[j]+1)] <- "E"
    ans[ ,j][(s[j]+2):(nc[j]+1)] <- "D"
    ans[ ,j][(su[j]+2):(nc[j]+1)] <- "DU"
    ans[ ,j][which(ans[, j] ==0)] <- "S"
    ind.s <- (1:(nc[k]+1)) > nc[j]+1
    ans[ , ][ind.s] <- rep("0", sum(ind.s))
  }
  out <- c(des, list(table=as.table(ans)))
  class(out) <- "dec.table"
  return(out)
}

#' Plot a decision table from a "dec.table" object
#' @description \code{plot} method for class \code{"dec.table"}.
#' @param x An object of class \code{"dec.table"}, typically returned by
#'   \code{\link{dec.table}}.
#' @param ... Unused arguments.
#' @details \code{plot.dec.table} plots the decision boundaries.
#' @import graphics
#' @export
#' @examples
#' truep <- c(0.3, 0.45, 0.5, 0.6)
#' out <- dec.table(0.6,0.4,0.2,0.3,c(3,3,3))
#' plot(out)
plot.dec.table <- function(x, ...) {
  n <- x$n
  nc <- cumsum(n)
  r <- x$E
  s <- x$D
  su <- x$DU
  col <- c("cadetblue1", "darkorchid2", "indianred2", "seagreen2")
  plot(nc, r, xaxt = "n", ylab = "Boundary", xlab = 'Sample Size', ylim = c(0, max(su)+2.3), type = "o", col=col[1], main = "Decision Plot", panel.first = grid())
  points(nc, s + 1, type = "o", col = col[2])
  points(nc, su + 1, type = "o", col = col[3])
  axis(1, at = nc, labels = nc)
  polygon(c(nc, rev(nc)), c(r + 0.05, rev(s + 1 - 0.05)), col = col[4], border = NA)
  polygon(c(nc, rev(nc)), c(rep(0,length(nc)), rev(r - 0.05)), col = col[1], border = NA)
  polygon(c(nc, rev(nc)), c(s + 1 + 0.05, rev(su + 1 - 0.05)), col = col[2], border = NA)
  polygon(c(nc, rev(nc)), c(su + 1 + 0.05, rev(rep(max(su)+2, length(nc)))), col = col[3], border = NA)
  legend("top", c("E (<=)", "D (>=)", "DU (>=)", "S"), horiz = TRUE, fill = col)
}


#' Print a decision table from a "dec.table" object
#' @description \code{print} method for class \code{"dec.table"}.
#' @param x An object of class \code{"dec.table"}, typically returned by
#'   \code{\link{dec.table}}.
#' @param ... Unused arguments.
#' @details \code{print.dec.table} prints the decision table together with a
#'   legend.
#' @export
#' @examples
#' print(dec.table(0.6,0.4,0.2,0.3,c(3,3,3)))
print.dec.table <- function(x, ...) {
  print(x$table)
  cat("\n")
  cat("	  Row : Number of DLTs", "\n")
  cat("Column : Number of subjects", "\n")
  cat("     E : Escalate to the next higher dose", "\n")
  cat("     S : Stay at the same dose", "\n")
  cat("     D : De-escalate to the previous lower dose", "\n")
  cat("    DU : De-escalate and never use this dose again", "\n")
  cat("\n")
  cat("Type 1 error : ", "\n")
  print(x$alpha.two)
  cat("Type 2 error : ", x$beta,"\n")
  cat("Right-side type 1 error : ", "\n")
  print(x$alpha.one)
}

#' Plot simulation results from a "dec.sim" object
#' @description Available plot types are: true toxicity at each dose level
#'   (\code{type = "s"}); a bar plot of the probability of selecting each dose
#'   as the MTD (\code{type = "prob"}); a bar plot of the average number of
#'   patients treated at each dose level (\code{type = "np"}); and a bar plot
#'   of the average number of DLTs at each dose level (\code{type = "dlt"}).
#'   Setting \code{type = "all"} produces all four plots.
#' @param x An object of class \code{"dec.sim"} or \code{"sl.sim"}, returned
#'   by \code{\link{dec.sim}} or \code{\link{sl.sim}}.
#' @param pt A vector of target toxicity values, one for each scenario.
#' @param s The scenario to plot. Defaults to 1.
#' @param type Plot type. See the description above.
#' @param label Logical; if \code{TRUE}, values are displayed on the plot.
#' @param col Graphical parameter \code{col}; see \code{\link{par}}.
#' @param text.col Color used for text labels.
#' @param cex Graphical parameter \code{cex}; see \code{\link{par}}.
#' @param ... Arguments passed to plotting functions.
#' @import graphics
#' @export
#' @examples 
#' # generate decision table
#' dt <- dec.table(0.6,0.4,0.2,0.3,c(3,3,3))
#' # Simulate trials from test data
#' test.file <- system.file("extdata", "testS.csv", package = "tsdf")
#' out <- sl.sim(dt$table, test.file)
#' plot(out, pt=rep(0.3,2), s=1, type="all")
#' plot(out, pt=rep(0.3,2), s=2, type="prob")
#' plot(out, pt=rep(0.3,2), s=1, type="np")
#' plot(out, pt=rep(0.3,2), s=2, type="dlt")
plot.dec.sim <- function(x, pt, s = 1, type = c("all", "s", "prob", "np", "dlt"), label = TRUE, col = "cornflowerblue", text.col = "darkblue", cex = 1, ...) {
  type <- match.arg(type)
  if (class(x)[1] == "sl.sim") {
    obj <- x[[s]]
    ns <- length(x)
  } else {
    obj <- x
    ns <- 1
  }
  truep <- obj$truep
  n_dose <- length(truep)
  names.arg <- 1:n_dose
  scen <- paste("( Scenario", s, ")")
  if (missing(pt)) {
    print("No target toxicity/mtd shown")
  }
  else if (length(pt) != ns) {
    stop("true toxicity is a vector with length = # of cenarios")
  } else {
    if(max(truep) < pt[s]) {
      print("MTD is higher than highest dose")
    } else if(min(truep) > pt[s]) {
      print("MTD is lower than lowest dose")
    } 
    else {
      mtd <- max(which(truep <= pt[s]))
      names.arg[mtd] <- paste(mtd, "(MTD)")
    }
  }
  if (type == "all") {
    par(mfrow = c(2, 2), cex = cex)
    plot(x, pt, s, "s", label, col, text.col, cex, ...)
    plot(x, pt, s, "prob", label, col, text.col, cex, ...)
    plot(x, pt, s, "dlt", label, col, text.col, cex, ...)
    plot(x, pt, s, "np", label, col, text.col, cex, ...)		
  }
  if (type == "prob") {
    ans <- obj$mtd.prob[1:n_dose]
    names(ans) <- NULL
    out <- barplot(rep(NA, n_dose), xlab = "Dose level", names.arg = names.arg, panel.first = box(), ylim = c(0, max(ans) * 1.1))
    grid()
    barplot(ans, add = TRUE, col = col)
    title(paste("Probability of selection", scen))
    if (label) {
      text(out, ans, labels = ans, pos = 3, cex = cex, col = text.col)
    }
  }
  if (type == "dlt") {
    ans <- obj$dlt
    out <- barplot(rep(NA, n_dose), xlab = "Dose level", names.arg = names.arg, panel.first = box(), ylim = c(0, max(ans) * 1.1))
    grid()
    barplot(ans, add = TRUE, col = col)
    title(paste("Average number of DLTs", scen))
    if (label) {
      text(out, ans, labels = ans, pos = 3, cex = cex, col = text.col)
    }
  }
  if (type == "np") {
    ans <- obj$n.patients
    out <- barplot(rep(NA, n_dose), xlab = "Dose level", names.arg = names.arg, panel.first = box(), ylim = c(0, max(ans) * 1.1))
    grid()
    barplot(ans, add = TRUE, col = col)
    title(paste("Average number of patients", scen))
    if (label) {
      text(out, ans, labels = ans, pos = 3, cex = cex, col = text.col)
    }
  }
  if (type == "s") {
    out <- plot(1:n_dose, truep, xlab = "Dose level", ylab = "Toxicity", col = col, type = "b", ylim = c(min(truep) * 0.9, max(truep) * 1.1), xaxt = "n", pch = 19, panel.first = c(box(), grid()))
    axis(1, at = 1:n_dose, labels = names.arg)
    title(paste("True toxicity", scen))
    if (!missing(pt)) {
      abline(pt[s], 0, lty = 5)
      text(n_dose/2, pt[s], labels = paste("Target =", pt[s]), pos = 3, cex = cex, col = text.col)
    }
    if (label) {
      text(1:n_dose, truep, labels = truep, pos = 3, cex = cex, col = text.col)
    }
  }
}


#' Summarize simulation results from a "dec.sim" object
#' @description \code{summary} method for class \code{"dec.sim"}.
#' @details \code{summary} formats key statistics from dose-finding
#'   simulations. Given the target toxicity, it reports the probability of
#'   selecting each dose level as the MTD, the probability of selecting doses
#'   above the true MTD, the probability of selecting the true MTD, and the
#'   probability that subjects are treated at or below the true MTD. The true
#'   MTD is defined as the highest dose level with toxicity probability less
#'   than or equal to the target toxicity. If the target is below the smallest
#'   toxicity probability, the lowest dose level is treated as the MTD. For
#'   example, if the target is 0.3 and the true toxicity probabilities for five
#'   doses are 0.1, 0.25, 0.35, 0.40, and 0.50, then the MTD is dose 2.
#' @param object An object of class \code{"dec.sim"}, returned by
#'   \code{\link{dec.sim}} or \code{\link{sl.sim}}.
#' @param pt Target toxicity for each scenario.
#' @param ... Unused arguments.
#' @examples 
#' test.file <- system.file("extdata", "testS.csv", package = "tsdf")
#' dt <- dec.table(0.6,0.4,0.2,0.3,c(3,3,3))
#' out <- sl.sim(dt$table, test.file)
#' pt <- c(0.3, 0.4)
#' summary(out, pt)
#' @import stats
#' @export
summary.dec.sim <- function(object, pt, ...) {
  if(missing(pt)) {
    warning("Missing true toxicity")
  }
  if(class(object)[1] == "sl.sim" & length(object) != length(pt)) {
    warning("pt length not equal to number of scenarios; only returns the first scenario stats")
  }
  ns <- length(pt)
  avg.np <- rep(0, ns)
  avg.dose <- rep(0, ns)
  avg.prob <- rep(0, ns)
  out <- vector("list", ns)
  for(i in 1:ns) {
    if(class(object)[1] == "sl.sim"){
      ans <- object[[i]]
    } else {
      ans <- object
    }
    truep <- ans$truep
    n_dose <- length(truep)
    res <- matrix(0, n_dose, 6)
    colnames(res) <- c("Level", "Truth", "MTD", "Over","DLT", "NP")
    res[, 1] <- 1:n_dose 
    res[, 2] <- truep
    res[, 3] <- ans$mtd.prob[1:n_dose]
    res[, 4] <- ans$over.prob[1:n_dose]
    res[, 5] <- ans$dlt
    res[, 6] <- ans$n.patients
    # calculate stats
    avg.np[i] <- sum(res[, 6])
    if(max(truep) < pt[i]) {
      mtd.dose <- "higher than highest dose"
      avg.dose[i] <- ans$mtd.prob[n_dose+2]
      avg.prob[i] <- 1
    } else if(min(truep) > pt[i]) {
      mtd.dose <- "lower than lowest dose"
      avg.dose[i] <- ans$mtd.prob[n_dose+1]
      avg.prob[i] <- 0
    } else {
      mtd.dose <- max(which(truep <= pt[i]))
      avg.dose[i] <- res[, 3][mtd.dose]
      avg.prob[i] <-sum(res[, 6][truep <= pt[i]])/sum(res[, 6])
    }
    out[[i]] <- list(dose.stats = res, prob.select = avg.dose[i], at.below.mtd = avg.prob[i], mtd = mtd.dose, nsim = ans$nsim, pt = pt[i], avg.np = avg.np[i], start.level = ans$start.level)
  }
  class(out) <- "summary.dec.sim"
  out
}

#' @export
print.summary.dec.sim <- function(x, ...) {
  cat("What does each column represent ?", "\n\n")
  cat("Level : Dose level", "\n")
  cat("Truth : True toxicity probability", "\n")
  cat("MTD   : The probability of selecting current dose level as the MTD", "\n")
  cat("Over  : The probability of selecting current dose level as over the MTD", "\n")
  cat("DLT   : The average number of subjects experienced DLT at current dose level", "\n")
  cat("NP    : The average number of subjects treated at current dose level", "\n\n")
  
  for(i in 1:length(x)){
    # table legends
    cat("Scenario ", i, "\n\n")
    cat("Simulation results are based on", x[[i]]$nsim, "simulated trials","\n")
    cat("Starting dose level is", x[[i]]$start.level, "; MTD is dose", x[[i]]$mtd, "\n")
    cat("Target toxicity probability =", x[[i]]$pt, "\n")
    cat("Average number of subjects =", x[[i]]$avg.np, "\n")
    cat("Probability of selecting the true MTD =", x[[i]]$prob.select, "\n")
    cat("Probability of subjects treated at or below the true MTD =", x[[i]]$at.below.mtd, "\n\n")
    print(as.data.frame(x[[i]]$dose.stats))
    cat("\n")
  }
}


#' Print Zhong's design from an "opt.design" object
#' @description \code{print} method for class \code{"opt.design"}.
#' @param x An object of class \code{"opt.design"}, typically returned by
#'   \code{\link{opt.design}}.
#' @param ... Unused arguments.
#' @export
#' @examples
#' alpha1 <- 0.20
#' alpha2 <- 0.1
#' beta <- 0.20
#' pc <- 0.5
#' pt <- pc + 0.2
#' out <- opt.design(alpha1, alpha2, beta, pc, pt, stage = 2, sf.param = 1)
#' print(out)
print.opt.design <- function(x, ...) {
  cat("\n Zhong's", x$stage,"stage Phase II design \n\n")
  cat("Minimal response rate: ", unique(x$pe), "\n")
  cat("Postulate response rate: ", x$pt, "\n")
  cat("Left-side type 1 error: ",x$alpha1, "\n")
  cat("Right-side type 1 error: ",x$alpha2, "\n")
  cat("Type 2 error: ",x$beta, "\n\n")
  cat("Notation:", "\n")
  cat("Left-side rejection region at stage i is response <= ri", "\n")
  cat("Right-side rejection region at stage", x$stage, "is response > s", "\n")
  cat("Sample size used at stage i is ni", "\n\n")
  cat("Optimal design : ", "\n")
  print(c(x$bdry, x$n))
  cat("True errors : ", "\n")
  print(x$error)
  cat("\n")
}

---
title: "Quick Start Guide"
author: "Wenchuan Guo and Bob Zhong"
date: '`r Sys.Date()`'
output:
  html_document:
    fig_height: 6
    fig_width: 7
    highlight: tango
    theme: united
  pdf_document: null
vignette: >
  %\VignetteIndexEntry{Quick Start Guide}
  %\VignetteEngine{knitr::rmarkdown}
---

This guide introduces the basic workflow of `tsdf`. For additional details, see the documentation for individual functions.

## Installation
Install the package from R with:

```{r, eval=FALSE}
install.packages("tsdf")
```

Then load the package:

```{r}
library(tsdf)
```

The remaining sections give short examples of the main functions.

## Zhong's 2-/3-stage Phase II designs
To calculate Zhong's two- or three-stage design, specify the left-side type I error (`alpha1`), right-side type I error (`alpha2`), type II error (`beta`), minimal response rate (`pc`), and postulated response rate (`pe`). The `stage` argument should be set to `2` or `3`.

The following example computes a two-stage design:

```{r}
# type I errors
alpha1 <- 0.15
alpha2 <- 0.10

# type II error
beta <- 0.15

# response rates
pc <- 0.25
pe <- pc + 0.20

# two-stage design
out <- opt.design(alpha1, alpha2, beta, pc, pe, stage = 2)
```

`out` is an object of class `opt.design`. The `print` method shows the recommended design and its operating characteristics:

```{r}
print(out)
```

Alpha-spending methods are available for both two- and three-stage designs. `opt.design()` uses the Hwang-Shih-DeCani spending function, with `sf.param` controlling the shape of the spending rule. For two-stage designs, the default is `NULL`, which means no alpha-spending is used. For three-stage designs, the default is `4`. For example, to use a Pocock-like spending function, set `sf.param = 1`:

```{r, eval=FALSE}
opt.design(alpha1, alpha2, beta, pc, pe, stage = 2, sf.param = 1)
```

## Decision table for Phase I dose-finding
`dec.table()` generates a decision table for a three-stage dose-finding design. `alpha.l` (left side), `alpha.r` (right side), and `alpha.u` (right-side type I error for `"DU"`, usually smaller than `alpha.r`) control the decision boundaries. `pt` is the target toxicity level and may be given as either a single value or an interval.

This procedure also supports alpha-spending through the `sf.param` argument, which has the same interpretation as in `opt.design()`. The default is `4`.

```{r}
# cohort sizes at each stage
n <- rep(3, 3)

# type I errors
alpha.l <- 0.6
alpha.r <- 0.4
alpha.u <- 0.2

# target toxicity
pt <- 0.3

# generate the decision table
out <- dec.table(alpha.l, alpha.r, alpha.u, pt, n)
```

`out` is an object of class `dec.table` that contains the decision table, realized error rates, and input settings. In most cases, it is better to use the provided S3 methods than to extract components manually.

View the decision table with:

```{r}
print(out)
```

Plot the decision boundaries with:

```{r}
plot(out)
```

## Dose-finding simulations
`dec.sim()` and `sl.sim()` run dose-finding simulations from a user-supplied decision table. To run a simulation, provide a decision table, the true toxicity probabilities for each dose level, the starting dose level (default: the lowest dose), and the number of simulated trials (default: `1000`).

`dec.sim()` runs a single scenario. `sl.sim()` runs a list of scenarios stored in a `.csv` or `.txt` file.

The simulation functions expect the decision table to be a matrix- or data-frame-like object with sample sizes as column names. If you start from `dec.table()`, convert `out$table` before passing it to the simulation functions:

```{r}
# true toxicity probabilities
truep <- c(0.3, 0.45, 0.5, 0.6)

# generate a decision table
dt <- dec.table(0.6, 0.4, 0.2, 0.3, c(3, 3, 3))

# convert the table object to a plain matrix for simulation
sim_table <- matrix(
  as.character(dt$table),
  nrow = nrow(dt$table),
  dimnames = dimnames(dt$table)
)

# run one simulation scenario
out1 <- dec.sim(truep, sim_table, start.level = 2, nsim = 1000)
```

The package also includes a sample scenario list:

```{r}
test.file <- system.file("extdata", "testS.csv", package = "tsdf")
```

Run simulations across all scenarios with:

```{r}
out2 <- sl.sim(sim_table, test.file)
```

`out1` is an object of class `dec.sim`. When the scenario file contains more than one scenario, `out2` has class `c("sl.sim", "dec.sim")`. Summary and plot methods are available for these objects. For example, the following command reports commonly used summary statistics:

```{r}
# target toxicity for each scenario
pt <- c(0.3, 0.4)
summary(out2, pt)
```

There are four plot types for a `dec.sim` object. The `s` argument selects the scenario to display, and `pt` gives the target toxicity for each scenario. See `?plot.dec.sim` for details:

```{r}
# scenario information (true toxicity)
plot(out2, s = 2, pt = c(0.3, 0.4), type = "s")

# probability of selecting each dose as the MTD
plot(out2, s = 2, pt = c(0.3, 0.4), type = "prob")

# average number of patients treated at each dose
plot(out2, s = 2, pt = c(0.3, 0.4), type = "np")

# average number of DLTs at each dose
plot(out2, s = 2, pt = c(0.3, 0.4), type = "dlt")
```

To display all plot types in a single figure, run:

```{r fig.height = 8}
plot(out2, pt = c(0.3, 0.4), type = "all", cex = 0.7)
```

## Scenario list and customized table format
The `.csv` or `.txt` files used by `sl.sim()` look like this:

```{r echo=FALSE, results='asis'}
sl <- system.file("extdata", "testS.csv", package = "tsdf")
knitr::kable(read.table(sl, header = TRUE, sep = ","))
```

This example is stored in `inst/extdata/testS.csv` and can be located with `system.file()`. The file should include:

- `Start Dose`: the starting dose level.
- `N Trials`: the number of simulated trials.
- `Dose*`: the true toxicity probability at each dose level.

If scenarios contain different numbers of dose levels, the raw file does not need explicit `NA` values; blank cells are sufficient.

Both `dec.sim()` and `sl.sim()` also support user-supplied decision tables. The table can be a `matrix`, `data.frame`, or similar rectangular object in R. The user is responsible for ensuring that the table is a valid dose-finding decision table. In particular, the column names should be the actual cumulative sample sizes rather than the default names created by `read.table()`, such as `X.*` or `V.*`.

For example, the following code reads the sample decision table included in the package:

```{r, warning=FALSE}
table.file <- system.file("extdata", "decTable.csv", package = "tsdf")
dec <- read.csv(table.file, row.names = 1, check.names = FALSE)
colnames(dec)
```

After importing the table, it should have the following format:

```{r, echo=FALSE}
knitr::kable(dec)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/opt.design.R
\name{opt.design}
\alias{opt.design}
\title{Zhong's 2- or 3-stage Phase II design}
\usage{
opt.design(
  alpha1,
  alpha2,
  beta,
  pc,
  pe,
  stage = 2,
  stop.eff = FALSE,
  frac_n1 = NULL,
  frac_n2 = NULL,
  sf.param = NULL,
  show = FALSE,
  nmax = 100,
  n.choice = 1,
  ...
)
}
\arguments{
\item{alpha1}{Left-side overall type I error.}

\item{alpha2}{right-side overall type I error.}

\item{beta}{Type II error.}

\item{pc}{A numeric vector of response rates. It should have length 1 or 2.}

\item{pe}{Alternative hypothesis response rate.}

\item{stage}{Either 2 or 3. Defaults to 2.}

\item{stop.eff}{Logical; if \code{TRUE}, the trial may stop early for
efficacy at an interim analysis.}

\item{frac_n1}{Proportion range for \code{n1}. For a two-stage design, the
default is \code{c(0.3, 0.6)}. For a three-stage design, the default is
\code{c(0.2, 0.3)}.}

\item{frac_n2}{Proportion range for \code{n2}. Used only for three-stage
designs. Defaults to \code{c(0.2, 0.4)}.}

\item{sf.param}{A single real value specifying the gamma parameter for the
Hwang-Shih-DeCani spending function. The allowable range is [-40, 40].
Larger values spend more error early and leave less for later stages. For
two-stage designs, the default is \code{NULL} (no alpha-spending). For
three-stage designs, the default is 4.}

\item{show}{Logical; if \code{TRUE}, the current total sample size is printed
during the search.}

\item{nmax}{Maximum sample size. Defaults to 100.}

\item{n.choice}{Stopping criterion for the search over feasible designs. The
search stops once the number of designs exceeds \code{n.choice}.}

\item{...}{Unused arguments.}
}
\value{
An object of class \code{"opt.design"}, returned as a list
  containing:
  \item{bdry}{The rejection boundaries.}
  \item{error}{The true type I and type II errors.}
  \item{n}{The sample size at each stage.}
  \item{complete}{The complete list of feasible designs.}
  \item{alpha1}{The input left-side type I error.}
  \item{alpha2}{The input right-side type I error.}
  \item{beta}{The input type II error.}
  \item{pc}{The input response-rate vector.}
  \item{pe}{The input alternative response rate.}
  \item{sf.param}{The input alpha-spending parameter.}
  \item{stage}{The number of stages in the selected design.}
}
\description{
Calculate the optimal 2- or 3-stage design proposed by Bob
  Zhong.
}
\details{
In the two-stage design, \code{n1} patients are treated in the
  first stage. At the end of stage 1, the trial either continues to stage 2
  or stops early for inefficacy, depending on the number of observed
  responses. If the trial continues, an additional \code{n2} patients are
  treated. The three-stage design extends the two-stage design by adding one
  interim stage between stages 1 and 2. The left-side rejection region is
  defined by \code{response <= r_i} for \code{i = 1, 2, 3}, and the
  right-side rejection region is defined by \code{response > s}. An
  alpha-spending method is available for both two- and three-stage designs.
  \code{opt.design} uses the Hwang-Shih-DeCani spending function; you can
  change the definition of \code{HSD} to use a different spending function.
}
\examples{
 alpha1 <- 0.15
 alpha2 <- 0.10
 beta <- 0.15
 pc <- 0.25
 pe <- pc + 0.20
 # calculate optimal two-stage design without using alpha-spending
 opt.design(alpha1, alpha2, beta, pc, pe, stage=2)
 
 # calculate optimal two-stage design with Pocock-like spending function 
 opt.design(alpha1, alpha2, beta, pc, pe, stage = 2, sf.param = 1)
 
 # calculate optimal three-stage design with an O'Brien-Fleming-like spending function
 opt.design(alpha1, alpha2, beta, pc, pe, stage = 3, sf.param = -4)
}
\references{
Zhong. (2012) Single-arm Phase IIA clinical trials with go/no-go decisions. \emph{Contemporary Clinical Trials}, \bold{33}, 1272--1279.
}
\author{
Wenchuan Guo <wguo1017@gmail.com>, Jianan Hui <jiananhuistat@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utility.R
\name{plot.dec.table}
\alias{plot.dec.table}
\title{Plot a decision table from a "dec.table" object}
\usage{
\method{plot}{dec.table}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"dec.table"}, typically returned by
\code{\link{dec.table}}.}

\item{...}{Unused arguments.}
}
\description{
\code{plot} method for class \code{"dec.table"}.
}
\details{
\code{plot.dec.table} plots the decision boundaries.
}
\examples{
truep <- c(0.3, 0.45, 0.5, 0.6)
out <- dec.table(0.6,0.4,0.2,0.3,c(3,3,3))
plot(out)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utility.R
\name{summary.dec.sim}
\alias{summary.dec.sim}
\title{Summarize simulation results from a "dec.sim" object}
\usage{
\method{summary}{dec.sim}(object, pt, ...)
}
\arguments{
\item{object}{An object of class \code{"dec.sim"}, returned by
\code{\link{dec.sim}} or \code{\link{sl.sim}}.}

\item{pt}{Target toxicity for each scenario.}

\item{...}{Unused arguments.}
}
\description{
\code{summary} method for class \code{"dec.sim"}.
}
\details{
\code{summary} formats key statistics from dose-finding
  simulations. Given the target toxicity, it reports the probability of
  selecting each dose level as the MTD, the probability of selecting doses
  above the true MTD, the probability of selecting the true MTD, and the
  probability that subjects are treated at or below the true MTD. The true
  MTD is defined as the highest dose level with toxicity probability less
  than or equal to the target toxicity. If the target is below the smallest
  toxicity probability, the lowest dose level is treated as the MTD. For
  example, if the target is 0.3 and the true toxicity probabilities for five
  doses are 0.1, 0.25, 0.35, 0.40, and 0.50, then the MTD is dose 2.
}
\examples{
test.file <- system.file("extdata", "testS.csv", package = "tsdf")
dt <- dec.table(0.6,0.4,0.2,0.3,c(3,3,3))
out <- sl.sim(dt$table, test.file)
pt <- c(0.3, 0.4)
summary(out, pt)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utility.R
\name{print.opt.design}
\alias{print.opt.design}
\title{Print Zhong's design from an "opt.design" object}
\usage{
\method{print}{opt.design}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"opt.design"}, typically returned by
\code{\link{opt.design}}.}

\item{...}{Unused arguments.}
}
\description{
\code{print} method for class \code{"opt.design"}.
}
\examples{
alpha1 <- 0.20
alpha2 <- 0.1
beta <- 0.20
pc <- 0.5
pt <- pc + 0.2
out <- opt.design(alpha1, alpha2, beta, pc, pt, stage = 2, sf.param = 1)
print(out)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sl.sim.R
\name{sl.sim}
\alias{sl.sim}
\title{Dose-finding simulations for a list of scenarios}
\usage{
sl.sim(decTable, file, header = TRUE, sep = ",", ...)
}
\arguments{
\item{decTable}{A customized decision table in the same format as the output
of \code{\link{dec.table}}.}

\item{file}{The name of the file containing the scenario definitions. See
\code{\link{read.table}} for details.}

\item{header}{Logical; if \code{TRUE}, the file contains variable names in
the first line. See \code{\link{read.table}} for details.}

\item{sep}{The field separator character. Defaults to \code{","}. See
\code{\link{read.table}} for details.}

\item{...}{Additional arguments passed to \code{\link{read.table}}.}
}
\value{
An object of class \code{"dec.sim"} (for one scenario) or
  \code{"sl.sim"} (for multiple scenarios). Use \code{\link{summary}} to
  obtain and print a summary of the results. The returned object is a list
  containing:
  \item{mtd}{A vector of dose levels giving the recommended maximum
  tolerated dose (MTD) at the end of each trial.}
  \item{n.patients}{The average number of patients treated at each dose
  level.}
  \item{truep}{The input true toxicity probabilities.}
  \item{start.level}{The input starting dose level.}
  \item{nsim}{The input number of simulated trials.}
}
\description{
Run dose-finding simulations based on a customized decision
  table for one or more scenarios read from a file.
}
\details{
In each row of the input file, the parameters must be ordered as
  \code{start.level}, \code{nsim}, and \code{truep}. The
  dose-finding algorithm is described in \code{\link{dec.sim}}.
}
\examples{
dt <- dec.table(0.6,0.4,0.2,0.3,c(3,3,3))
test.file <- system.file("extdata", "testS.csv", package = "tsdf")
# use a customized decision table
table.file <- system.file("extdata", "decTable.csv", package = "tsdf")
dec <- read.csv(table.file, row.names = 1, check.names = FALSE)
out1 <- sl.sim(as.matrix(dt$table), test.file)
out2 <- sl.sim(dec, test.file)
}
\author{
Wenchuan Guo <wguo1017@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dec.table.R
\name{dec.table}
\alias{dec.table}
\title{Generate a dose-finding decision table}
\usage{
dec.table(alpha.l, alpha.r, alpha.u, pt, n, sf.param = 4, pe.par = 0.25, ...)
}
\arguments{
\item{alpha.l}{Left-side overall type I error. This controls the upper bound
for dose escalation.}

\item{alpha.r}{Right-side overall type I error. This controls the lower bound
for dose de-escalation.}

\item{alpha.u}{Right-side overall type I error used to determine the lower
bound for \code{"DU"}.}

\item{pt}{A numeric vector of target toxicities. It should have length 1, or
length 2 when the target is an interval.}

\item{n}{A vector of sample sizes at each stage. \code{sum(n)} is the total
sample size. For A+B designs, \code{n} has length 2; for A+B+C designs,
\code{n} has length 3.}

\item{sf.param}{A single real value specifying the gamma parameter for the
Hwang-Shih-DeCani spending function. The allowable range is [-40, 40].
Larger values spend more error early and leave less for later stages.
Defaults to 4.}

\item{pe.par}{Alternative hypothesis offset used to calculate power and the
type II error. The alternative is defined as \code{pe = pt + pe.par}.
Defaults to 0.25.}

\item{...}{Unused arguments.}
}
\value{
An object of class \code{"dec.table"}, returned as a list
  containing:
  \item{table}{The generated decision table.}
  \item{alpha.two}{A vector of true type I errors for the two-sided test.}
  \item{alpha.one}{A vector of true type I errors for the right-sided test.}
  \item{beta}{The true type II error, which depends on the alternative
  hypothesis.}
  \item{E}{A vector of \code{"E"} boundaries.}
  \item{D}{A vector of \code{"D"} boundaries.}
  \item{DU}{A vector of \code{"DU"} boundaries.}
  \item{pt}{The input target toxicity vector.}
  \item{n}{The input stage-wise sample sizes.}
  \item{sf.param}{The input alpha-spending parameter.}
}
\description{
Generate a two- or three-stage dose-finding decision table.
}
\details{
An alpha-spending method is available for two- and three-stage
  designs. \code{dec.table} uses the Hwang-Shih-DeCani spending function.
}
\examples{
alpha.l <- 0.6
alpha.r <- 0.4
alpha.u <- 0.1
pt <- 0.3
# Print the decision table for a 3+3+3 design
n <- rep(3, 3)
dec.table(alpha.l, alpha.r, alpha.u, pt, n)$table
# 3+3 design
n <- rep(3, 2)
dec.table(alpha.l, alpha.r, alpha.u, pt, n)$table
}
\author{
Wenchuan Guo <wguo1017@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adj.two.R
\name{adj.two}
\alias{adj.two}
\title{Adjust Zhong's two-stage Phase II design}
\usage{
adj.two(n1, r1, s1, n2, alpha1, alpha2, beta, pc, pe, ...)
}
\arguments{
\item{n1}{Sample size at stage 1.}

\item{r1}{Inefficacy boundary at stage 1.}

\item{s1}{Efficacy boundary at stage 1. If there is no early stopping for
efficacy, \code{s1} should equal \code{n1}.}

\item{n2}{Sample size at stage 2.}

\item{alpha1}{Left-side overall type I error.}

\item{alpha2}{right-side overall type I error.}

\item{beta}{Type II error.}

\item{pc}{A numeric vector of response rates. It should have length 1 or 2.}

\item{pe}{Alternative hypothesis response rate.}

\item{...}{Unused arguments.}
}
\value{
An object of class \code{"opt.design"}, returned as a list
  containing:
  \item{bdry}{The rejection boundaries.}
  \item{error}{The true type I and type II errors.}
  \item{n}{The sample size at each stage.}
  \item{complete}{The complete list of feasible designs.}
  \item{alpha1}{The input left-side type I error.}
  \item{alpha2}{The input right-side type I error.}
  \item{beta}{The input type II error.}
  \item{pc}{The input response-rate vector.}
  \item{pe}{The input alternative response rate.}
  \item{stage}{The number of stages in the selected design.}
}
\description{
Adjust Zhong's two-stage design for over-running or
  under-running.
}
\details{
This function enumerates feasible second-stage boundaries after the
  first-stage sample size and boundaries have been fixed.
}
\examples{
 n1 <- 22
 r1 <- 6
 s1 <- 22
 n2 <- 24
 pc <- 0.4
 pe <- pc + 0.15
 alpha1 <- 0.3
 alpha2 <- 0.1
 beta <- 0.2
 out <- adj.two(n1, r1, s1, n2, alpha1, alpha2, beta, pc, pe)
}
\author{
Wenchuan Guo <wguo1017@gmail.com>, Jianan Hui <jiananhuistat@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dec.sim.R
\name{dec.sim}
\alias{dec.sim}
\title{Run dose-finding simulations}
\usage{
dec.sim(truep, decTable, start.level = 1, nsim = 1000)
}
\arguments{
\item{truep}{A vector of length \eqn{k}, giving the true toxicity
probabilities for the dose levels under study.}

\item{decTable}{A customized decision table in the same format as the output
of \code{\link{dec.table}}.}

\item{start.level}{Starting dose level. Defaults to 1, the lowest dose
level.}

\item{nsim}{Number of simulated trials. Defaults to 1000.}
}
\value{
An object of class \code{"dec.sim"}. Use
  \code{\link{summary.dec.sim}} to obtain and print a summary table of the
  results. The returned object is a list containing:
  \item{mtd}{A vector of dose levels giving the recommended maximum
  tolerated dose (MTD) at the end of the trial.}
  \item{mtd.prob}{A vector of length \eqn{k} giving the proportion of trials
  that selected each dose level as the MTD.}
  \item{over.prob}{A vector of length \eqn{k} giving the proportion of
  trials that selected each dose level as above the MTD.}
  \item{n.patients}{The average number of patients treated at each dose
  level.}
  \item{dlt}{The average number of DLTs observed at each dose level.}
  \item{truep}{The input true toxicity probabilities.}
  \item{start.level}{The input starting dose level.}
  \item{nsim}{The input number of simulated trials.}
}
\description{
Run dose-finding simulations based on a customized decision
  table.
}
\details{
Assume there are \eqn{d} dose levels to be studied. Let
  \eqn{n_i} and \eqn{m_i} denote the cumulative number of patients treated
  and the cumulative number of DLTs observed at the current dose level,
  respectively. Let \eqn{n_{max}} be the maximum number of patients allowed
  at each dose level. The procedure is as follows:
\describe{
\item{Step 1}{Update the cumulative numbers of DLTs (\eqn{m_i}) and
total treated patients (\eqn{n_i}) at the current dose, then use the
decision table to choose an action. If the decision is \code{"S"}, go to
Step 2. If the decision is \code{"D"} or \code{"DU"}, go to Step 3. If the
decision is \code{"E"}, go to Step 4.}
\item{Step 2}{If \eqn{n_i = n_{max}}, declare dose \eqn{i} as the MTD.
Otherwise, treat an additional cohort at the current dose and return to
Step 1.}
\item{Step 3}{If the current dose is the lowest dose, stop the trial and
declare that the MTD is below the lowest dose. Otherwise, if the next lower
dose has not yet reached \eqn{n_{max}}, treat an additional cohort there,
move to that dose, and return to Step 1. If the next lower dose has already
reached \eqn{n_{max}}, stop the trial and declare that dose as the MTD. If
the decision is \code{"DU"}, record the current dose as unusable and do not
treat additional patients there.}
\item{Step 4}{If the current dose is the highest dose, stop the trial and
declare that the MTD is above the highest dose. Otherwise, if the next
higher dose has been marked \code{"DU"}, continue treating the current dose
until it reaches \eqn{n_{max}}; if that limit is reached, declare the
current dose as the MTD. If the next higher dose is available and has not
yet reached \eqn{n_{max}}, treat an additional cohort there, move up to
that dose, and return to Step 1. Otherwise, declare the current dose as the
MTD.}
}
}
\examples{
truep <- c(0.3, 0.45, 0.5, 0.6)
res <- dec.table(0.6,0.4,0.2,0.3,c(3,3,3))
out <- dec.sim(truep, as.matrix(res$table), start.level = 2, nsim = 1000)
summary(out, pt = 0.3)
}
\author{
Wenchuan Guo <wguo1017@gmail.com>
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utility.R
\name{plot.dec.sim}
\alias{plot.dec.sim}
\title{Plot simulation results from a "dec.sim" object}
\usage{
\method{plot}{dec.sim}(
  x,
  pt,
  s = 1,
  type = c("all", "s", "prob", "np", "dlt"),
  label = TRUE,
  col = "cornflowerblue",
  text.col = "darkblue",
  cex = 1,
  ...
)
}
\arguments{
\item{x}{An object of class \code{"dec.sim"} or \code{"sl.sim"}, returned
by \code{\link{dec.sim}} or \code{\link{sl.sim}}.}

\item{pt}{A vector of target toxicity values, one for each scenario.}

\item{s}{The scenario to plot. Defaults to 1.}

\item{type}{Plot type. See the description above.}

\item{label}{Logical; if \code{TRUE}, values are displayed on the plot.}

\item{col}{Graphical parameter \code{col}; see \code{\link{par}}.}

\item{text.col}{Color used for text labels.}

\item{cex}{Graphical parameter \code{cex}; see \code{\link{par}}.}

\item{...}{Arguments passed to plotting functions.}
}
\description{
Available plot types are: true toxicity at each dose level
  (\code{type = "s"}); a bar plot of the probability of selecting each dose
  as the MTD (\code{type = "prob"}); a bar plot of the average number of
  patients treated at each dose level (\code{type = "np"}); and a bar plot
  of the average number of DLTs at each dose level (\code{type = "dlt"}).
  Setting \code{type = "all"} produces all four plots.
}
\examples{
# generate decision table
dt <- dec.table(0.6,0.4,0.2,0.3,c(3,3,3))
# Simulate trials from test data
test.file <- system.file("extdata", "testS.csv", package = "tsdf")
out <- sl.sim(dt$table, test.file)
plot(out, pt=rep(0.3,2), s=1, type="all")
plot(out, pt=rep(0.3,2), s=2, type="prob")
plot(out, pt=rep(0.3,2), s=1, type="np")
plot(out, pt=rep(0.3,2), s=2, type="dlt")
}
